#include "edac.h"

/*
 * Some code have been modified from Robert's code 
 * 
 * File:    bch3.c
 * Title:   Encoder/decoder for binary BCH codes in C (Version 3.1)
 * Author:  Robert Morelos-Zaragoza
 * Date:    August 1994
 * Revised: June 13, 1997
 *
 * ===============  Encoder/Decoder for binary BCH codes in C =================
 *
 * Version 1:   Original program. The user provides the generator polynomial
 *              of the code (cumbersome!).
 * Version 2:   Computes the generator polynomial of the code.
 * Version 3:   No need to input the coefficients of a primitive polynomial of
 *              degree m, used to construct the Galois Field GF(2**m). The
 *              program now works for any binary BCH code of length such that:
 *              2**(m-1) - 1 < length <= 2**m - 1
 *
 * Note:        You may have to change the size of the arrays to make it work.
 *
 * The encoding and decoding methods used in this program are based on the
 * book "Error Control Coding: Fundamentals and Applications", by Lin and
 * Costello, Prentice Hall, 1983.
 *
 * Thanks to Patrick Boyle (pboyle@era.com) for his observation that 'bch2.c'
 * did not work for lengths other than 2**m-1 which led to this new version.
 * Portions of this program are from 'rs.c', a Reed-Solomon encoder/decoder
 * in C, written by Simon Rockliff (simon@augean.ua.oz.au) on 21/9/89. The
 * previous version of the BCH encoder/decoder in C, 'bch2.c', was written by
 * Robert Morelos-Zaragoza (robert@spectra.eng.hawaii.edu) on 5/19/92.
 *
 * NOTE:    
 *          The author is not responsible for any malfunctioning of
 *          this program, nor for any damage caused by it. Please include the
 *          original program along with these comments in any redistribution.
 *
 *  For more information, suggestions, or other ideas on implementing error
 *  correcting codes, please contact me at:
 *
 *                           Robert Morelos-Zaragoza
 *                           5120 Woodway, Suite 7036
 *                           Houston, Texas 77056
 *
 *                    email: r.morelos-zaragoza@ieee.org
 *
 * COPYRIGHT NOTICE: This computer program is free for non-commercial purposes.
 * You may implement this program for any non-commercial application. You may 
 * also implement this program for commercial purposes, provided that you
 * obtain my written permission. Any modification of this program is covered
 * by this copyright.
 *
 * == Copyright (c) 1994-7,  Robert Morelos-Zaragoza. All rights reserved.  ==
 *
 * m = order of the Galois field GF(2**m) 
 * n = 2**m - 1 = size of the multiplicative group of GF(2**m)
 * length = length of the BCH code
 * t = error correcting capability (max. no. of errors the code corrects)
 * d = 2*t + 1 = designed min. distance = no. of consecutive roots of g(x) + 1
 * k = n - deg(g(x)) = dimension (no. of information bits/codeword) of the code
 * p[] = coefficients of a primitive polynomial used to generate GF(2**m)
 * g[] = coefficients of the generator polynomial, g(x)
 * alpha_to [] = log table of GF(2**m) 
 * index_of[] = antilog table of GF(2**m)
 * data[] = information bits = coefficients of data polynomial, i(x)
 * bb[] = coefficients of redundancy polynomial x^(length-k) i(x) modulo g(x)
 * numerr = number of errors 
 * errpos[] = error positions 
 * recd[] = coefficients of the received polynomial 
 * decerror = number of decoding errors (in _message_ positions) 
 *
 */

void edac_init(void){
	//encoded_array_struct = pvPortMalloc(sizeof(struct EncodedElement)); 
	// To free on exit requires complex heap management
	
	decerror = 0;
	
	uint8_t	data_init[100] = {1,0,1,1,0,0,1,0,0,0,0,1,0,1,1,1,1,0,1,0,0,0,1,0,1,0,1,1,1,0,0,1,0,1,0,1,0,0,1,1,1,1,0,1,1,1,1,0,1,0,0,0,0}; //changed bit 53 from 0 to 1
	uint8_t i;
	for(i=0;i<100;i++){
		data_input[i] = data_init[i];
	}
	
}

void edac_read_p(void)
/*
 *	Read m, the degree of a primitive polynomial p(x) used to compute the
 *	Galois field GF(2**m). Get precomputed coefficients p[] of p(x). Read
 *	the code length.
 */
{
	int			i, ninf;
	m=7;
	for (i=1; i<m; i++)
		p[i] = 0;
	p[0] = p[m] = 1;
	if (m == 2)			p[1] = 1;
	else if (m == 3)	p[1] = 1;
	else if (m == 4)	p[1] = 1;
	else if (m == 5)	p[2] = 1;
	else if (m == 6)	p[1] = 1;
	else if (m == 7)	p[1] = 1;
	else if (m == 8)	p[4] = p[5] = p[6] = 1;
	else if (m == 9)	p[4] = 1;
	else if (m == 10)	p[3] = 1;
	else if (m == 11)	p[2] = 1;
	else if (m == 12)	p[3] = p[4] = p[7] = 1;
	else if (m == 13)	p[1] = p[3] = p[4] = 1;
	else if (m == 14)	p[1] = p[11] = p[12] = 1;
	else if (m == 15)	p[1] = 1;
	else if (m == 16)	p[2] = p[3] = p[5] = 1;
	else if (m == 17)	p[3] = 1;
	else if (m == 18)	p[7] = 1;
	else if (m == 19)	p[1] = p[5] = p[6] = 1;
	else if (m == 20)	p[3] = 1;
	
    n = 1;
	for (i = 0; i <= m; i++) {
        n *= 2;
        }
	n = n / 2 - 1;
	ninf = (n + 1) / 2 - 1;

	length=67;
}


void edac_generate_gf(void)
/*
 * Generate field GF(2**m) from the irreducible polynomial p(X) with
 * coefficients in p[0]..p[m].
 *
 * Lookup tables:
 *   index->polynomial form: alpha_to[] contains j=alpha^i;
 *   polynomial form -> index form:	index_of[j=alpha^i] = i
 *
 * alpha=2 is the primitive element of GF(2**m) 
 */
{
	int    i, mask;

	mask = 1;
	alpha_to[m] = 0;
	for (i = 0; i < m; i++) {
		alpha_to[i] = mask;
		index_of[alpha_to[i]] = i;
		if (p[i] != 0)
			alpha_to[m] ^= mask;
		mask <<= 1;
	}
	index_of[alpha_to[m]] = m;
	mask >>= 1;
	for (i = m + 1; i < n; i++) {
		if (alpha_to[i - 1] >= mask)
			alpha_to[i] = alpha_to[m] ^ ((alpha_to[i - 1] ^ mask) << 1);
		else
			alpha_to[i] = alpha_to[i - 1] << 1;
		index_of[alpha_to[i]] = i;
	}
	index_of[0] = -1;
	
}

void edac_gen_poly(void)
/*
 * Compute the generator polynomial of a binary BCH code. Fist generate the
 * cycle sets modulo 2**m - 1, cycle[][] =  (i, 2*i, 4*i, ..., 2^l*i). Then
 * determine those cycle sets that contain integers in the set of (d-1)
 * consecutive integers {1..(d-1)}. The generator polynomial is calculated
 * as the product of linear factors of the form (x+alpha^i), for every i in
 * the above cycle sets.
 */
{
	static int	ii, jj, ll, kaux;
	static int				test, aux, nocycles, root, noterms, rdncy;
	static int             cycle[100][21], size[100], min[100], zeros[100];
	
	/* Generate cycle sets modulo n, n = 2**m - 1 */
	cycle[0][0] = 0;
	size[0] = 1;
	cycle[1][0] = 1;
	size[1] = 1;
	jj = 1;			/* cycle set index */

	do {
		/* Generate the jj-th cycle set */
		ii = 0;
		do {
			ii++;
			cycle[jj][ii] = (cycle[jj][ii - 1] * 2) % n;
			size[jj]++;
			aux = (cycle[jj][ii] * 2) % n;
		} while (aux != cycle[jj][0]);
		/* Next cycle set representative */
		ll = 0;
		do {
			ll++;
			test = 0;
			for (ii = 1; ((ii <= jj) && (!test)); ii++)	
			/* Examine previous cycle sets */
			  for (kaux = 0; ((kaux < size[ii]) && (!test)); kaux++)
			     if (ll == cycle[ii][kaux])
			        test = 1;
		} while ((test) && (ll < (n - 1)));
		if (!(test)) {
			jj++;	/* next cycle set index */
			cycle[jj][0] = ll;
			size[jj] = 1;
		}
	} while (ll < (n - 1));
	nocycles = jj;		/* number of cycle sets modulo n */
	
	// t and d are global
	t = 2;
	d = 2 * t + 1;

	/* Search for roots 1, 2, ..., d-1 in cycle sets */
	kaux = 0;
	rdncy = 0;
	for (ii = 1; ii <= nocycles; ii++) {
		min[kaux] = 0;
		test = 0;
		for (jj = 0; ((jj < size[ii]) && (!test)); jj++)
			for (root = 1; ((root < d) && (!test)); root++)
				if (root == cycle[ii][jj])  {
					test = 1;
					min[kaux] = ii;
				}
		if (min[kaux]) {
			rdncy += size[min[kaux]];
			kaux++;
		}
	}

	noterms = kaux;
	kaux = 1;
	for (ii = 0; ii < noterms; ii++)
		for (jj = 0; jj < size[min[ii]]; jj++) {
			zeros[kaux] = cycle[min[ii]][jj];
			kaux++;
		}
	k = length - rdncy;
	
	
    if (k<0)
      {
         printf("Parameters invalid!\n");
		 return;
      }

	/* Compute the generator polynomial */
	g[0] = alpha_to[zeros[1]];
	g[1] = 1;		/* g(x) = (X + zeros[1]) initially */
	for (ii = 2; ii <= rdncy; ii++) {
	  g[ii] = 1;
	  for (jj = ii - 1; jj > 0; jj--)
	    if (g[jj] != 0)
	      g[jj] = g[jj - 1] ^ alpha_to[(index_of[g[jj]] + zeros[ii]) % n];
	    else
	      g[jj] = g[jj - 1];
	  g[0] = alpha_to[(index_of[g[0]] + zeros[ii]) % n];
	}
}


void edac_encode_bch(void)
/*
 * Compute redundacy bb[], the coefficients of b(x). The redundancy
 * polynomial b(x) is the remainder after dividing x^(length-k)*data(x)
 * by the generator polynomial g(x).
 */
{	
	int8_t    i, j;
	int16_t    feedback;

	for (i = 0; i < length - k; i++)
		bb[i] = 0;
	for (i = k - 1; i >= 0; i--) {
		feedback = data[i] ^ bb[length - k - 1];
		if (feedback != 0) {
			for (j = length - k - 1; j > 0; j--)
				if (g[j] != 0)
					bb[j] = bb[j - 1] ^ feedback;
				else
					bb[j] = bb[j - 1];
			bb[0] = g[0] && feedback;
		} else {
			for (j = length - k - 1; j > 0; j--)
				bb[j] = bb[j - 1];
			bb[0] = 0;
		}
	}
}



void edac_decode_bch(void)
/*
 * Simon Rockliff's implementation of Berlekamp's algorithm.
 *
 * Assume we have received bits in recd[i], i=0..(n-1).
 *
 * Compute the 2*t syndromes by substituting alpha^i into rec(X) and
 * evaluating, storing the syndromes in s[i], i=1..2t (leave s[0] zero) .
 * Then we use the Berlekamp algorithm to find the error location polynomial
 * elp[i].
 *
 * If the degree of the elp is >t, then we cannot correct all the errors, and
 * we have detected an uncorrectable error pattern. We output the information
 * bits uncorrected.
 *
 * If the degree of elp is <=t, we substitute alpha^i , i=1..n into the elp
 * to get the roots, hence the inverse roots, the error location numbers.
 * This step is usually called "Chien's search".
 *
 * If the number of errors located is not equal the degree of the elp, then
 * the decoder assumes that there are more than t errors and cannot correct
 * them, only detect them. We output the information bits uncorrected.
 */
{

	uint8_t    i, j, u, q, t2, count = 0, syn_error = 0;
	int8_t             elp[65][64], d[64], l[64], u_lu[64], s[64];
	uint8_t             root[40], loc[40], err[64], reg[40];
	
	t2 = 2 * t;

	/* first form the syndromes */
	for (i = 1; i <= t2; i++) {
		s[i] = 0;
		for (j = 0; j < length; j++)
			if (recd[j] != 0)
				s[i] ^= alpha_to[(i * j) % n];
		if (s[i] != 0)
			syn_error = 1; /* set error flag if non-zero syndrome */

		s[i] = index_of[s[i]];
	}

	if (syn_error) {	/* if there are errors, try to correct them */
		/*
		 * Compute the error location polynomial via the Berlekamp
		 * iterative algorithm. Following the terminology of Lin and
		 * Costello's book :   d[u] is the 'mu'th discrepancy, where
		 * u='mu'+1 and 'mu' (the Greek letter!) is the step number
		 * ranging from -1 to 2*t (see L&C),  l[u] is the degree of
		 * the elp at that step, and u_l[u] is the difference between
		 * the step number and the degree of the elp. 
		 */
		/* initialise table entries */
		d[0] = 0;			/* index form */
		d[1] = s[1];		/* index form */
		elp[0][0] = 0;		/* index form */
		elp[1][0] = 1;		/* polynomial form */
		for (i = 1; i < t2; i++) {
			elp[0][i] = -1;	/* index form */
			elp[1][i] = 0;	/* polynomial form */
		}
		l[0] = 0;
		l[1] = 0;
		u_lu[0] = -1;
		u_lu[1] = 0;
		u = 0;
 
		do {
			u++;
			if (d[u] == -1) {
				l[u + 1] = l[u];
				for (i = 0; i <= l[u]; i++) {
					elp[u + 1][i] = elp[u][i];
					elp[u][i] = index_of[elp[u][i]];
				}
			} else
				/*
				 * search for words with greatest u_lu[q] for
				 * which d[q]!=0 
				 */
			{
				q = u - 1;
				while ((d[q] == -1) && (q > 0))
					q--;
				/* have found first non-zero d[q]  */
				if (q > 0) {
				  j = q;
				  do {
				    j--;
				    if ((d[j] != -1) && (u_lu[q] < u_lu[j]))
				      q = j;
				  } while (j > 0);
				}
 
				/*
				 * have now found q such that d[u]!=0 and
				 * u_lu[q] is maximum 
				 */
				/* store degree of new elp polynomial */
				if (l[u] > l[q] + u - q)
					l[u + 1] = l[u];
				else
					l[u + 1] = l[q] + u - q;
 
				/* form new elp(x) */
				for (i = 0; i < t2; i++)
					elp[u + 1][i] = 0;
				for (i = 0; i <= l[q]; i++)
					if (elp[q][i] != -1)
						elp[u + 1][i + u - q] = 
                                   alpha_to[(d[u] + n - d[q] + elp[q][i]) % n];
				for (i = 0; i <= l[u]; i++) {
					elp[u + 1][i] ^= elp[u][i];
					elp[u][i] = index_of[elp[u][i]];
				}
			}
			u_lu[u + 1] = u - l[u + 1];
 
			/* form (u+1)th discrepancy */
			if (u < t2) {	
			/* no discrepancy computed on last iteration */
			  if (s[u + 1] != -1)
			    d[u + 1] = alpha_to[s[u + 1]];
			  else
			    d[u + 1] = 0;
			    for (i = 1; i <= l[u + 1]; i++)
			      if ((s[u + 1 - i] != -1) && (elp[u + 1][i] != 0))
			        d[u + 1] ^= alpha_to[(s[u + 1 - i] 
			                      + index_of[elp[u + 1][i]]) % n];
			  /* put d[u+1] into index form */
			  d[u + 1] = index_of[d[u + 1]];	
			}
		} while ((u < t2) && (l[u + 1] <= t));
 
		u++;
		if (l[u] <= t) {/* Can correct errors */
			/* put elp into index form */
			for (i = 0; i <= l[u]; i++)
				elp[u][i] = index_of[elp[u][i]];

			/* Chien search: find roots of the error location polynomial */
			for (i = 1; i <= l[u]; i++)
				reg[i] = elp[u][i];
			count = 0;
			for (i = 1; i <= n; i++) {
				q = 1;
				for (j = 1; j <= l[u]; j++)
					if (reg[j] != -1) {
						reg[j] = (reg[j] + j) % n;
						q ^= alpha_to[reg[j]];
					}
				if (!q) {	/* store root and error
						 * location number indices */
					root[count] = i;
					loc[count] = n - i;
					count++;
				}
			if (count == l[u])		
			/* no. roots = degree of elp hence <= t errors */
				for (i = 0; i < l[u]; i++)
					recd[loc[i]] ^= 1;
			else	/* elp has degree >t hence cannot solve */
				{}
		}
	}
}

void edac_gererate_errors(void){
	//generated with external script
	errors[0][0]=3;
	errors[0][1]=30;
	errors[0][2]=0;
	errors[0][3]=16;
	errors[0][4]=65;

	errors[1][0]=2;
	errors[1][1]=13;
	errors[1][2]=55;
	errors[1][3]=64;
	errors[1][4]=45;

	errors[2][0]=2;
	errors[2][1]=22;
	errors[2][2]=39;
	errors[2][3]=25;
	errors[2][4]=35;

	errors[3][0]=2;
	errors[3][1]=24;
	errors[3][2]=53;
	errors[3][3]=17;
	errors[3][4]=33;

	errors[4][0]=2;
	errors[4][1]=13;
	errors[4][2]=12;
	errors[4][3]=20;
	errors[4][4]=14;

	errors[5][0]=2;
	errors[5][1]=6;
	errors[5][2]=30;
	errors[5][3]=33;
	errors[5][4]=44;

	errors[6][0]=2;
	errors[6][1]=57;
	errors[6][2]=60;
	errors[6][3]=52;
	errors[6][4]=44;

	errors[7][0]=2;
	errors[7][1]=36;
	errors[7][2]=15;
	errors[7][3]=32;
	errors[7][4]=30;

	errors[8][0]=2;
	errors[8][1]=5;
	errors[8][2]=50;
	errors[8][3]=17;
	errors[8][4]=33;

	errors[9][0]=2;
	errors[9][1]=46;
	errors[9][2]=62;
	errors[9][3]=46;
	errors[9][4]=32;

	/*
	errors[10][0]=2;
	errors[10][1]=16;
	errors[10][2]=42;
	errors[10][3]=25;
	errors[10][4]=31;

	errors[11][0]=2;
	errors[11][1]=48;
	errors[11][2]=20;
	errors[11][3]=34;
	errors[11][4]=7;

	errors[12][0]=2;
	errors[12][1]=24;
	errors[12][2]=62;
	errors[12][3]=35;
	errors[12][4]=56;

	errors[13][0]=2;
	errors[13][1]=56;
	errors[13][2]=16;
	errors[13][3]=21;
	errors[13][4]=44;

	errors[14][0]=2;
	errors[14][1]=35;
	errors[14][2]=37;
	errors[14][3]=61;
	errors[14][4]=6;

	errors[15][0]=2;
	errors[15][1]=33;
	errors[15][2]=31;
	errors[15][3]=63;
	errors[15][4]=32;

	errors[16][0]=2;
	errors[16][1]=50;
	errors[16][2]=52;
	errors[16][3]=29;
	errors[16][4]=8;

	errors[17][0]=2;
	errors[17][1]=21;
	errors[17][2]=5;
	errors[17][3]=21;
	errors[17][4]=11;

	errors[18][0]=2;
	errors[18][1]=20;
	errors[18][2]=21;
	errors[18][3]=29;
	errors[18][4]=27;
	
	errors[19][0]=2;
	errors[19][1]=27;
	errors[19][2]=21;
	errors[19][3]=42;
	errors[19][4]=16;
	
	
	errors[20][0]=2;
	errors[20][1]=41;
	errors[20][2]=10;
	errors[20][3]=1;
	errors[20][4]=52;

	errors[21][0]=2;
	errors[21][1]=41;
	errors[21][2]=32;
	errors[21][3]=51;
	errors[21][4]=17;

	errors[22][0]=2;
	errors[22][1]=44;
	errors[22][2]=44;
	errors[22][3]=17;
	errors[22][4]=25;

	errors[23][0]=2;
	errors[23][1]=26;
	errors[23][2]=32;
	errors[23][3]=30;
	errors[23][4]=46;

	errors[24][0]=2;
	errors[24][1]=48;
	errors[24][2]=24;
	errors[24][3]=48;
	errors[24][4]=6;

	errors[25][0]=2;
	errors[25][1]=40;
	errors[25][2]=34;
	errors[25][3]=49;
	errors[25][4]=18;

	errors[26][0]=2;
	errors[26][1]=14;
	errors[26][2]=62;
	errors[26][3]=61;
	errors[26][4]=65;

	errors[27][0]=2;
	errors[27][1]=46;
	errors[27][2]=65;
	errors[27][3]=35;
	errors[27][4]=38;

	errors[28][0]=2;
	errors[28][1]=27;
	errors[28][2]=32;
	errors[28][3]=64;
	errors[28][4]=53;

	errors[29][0]=2;
	errors[29][1]=18;
	errors[29][2]=5;
	errors[29][3]=46;
	errors[29][4]=9;

	errors[30][0]=2;
	errors[30][1]=21;
	errors[30][2]=6;
	errors[30][3]=28;
	errors[30][4]=44;

	errors[31][0]=2;
	errors[31][1]=38;
	errors[31][2]=49;
	errors[31][3]=11;
	errors[31][4]=7;

	errors[32][0]=2;
	errors[32][1]=59;
	errors[32][2]=35;
	errors[32][3]=60;
	errors[32][4]=43;

	errors[33][0]=2;
	errors[33][1]=25;
	errors[33][2]=11;
	errors[33][3]=48;
	errors[33][4]=25;

	errors[34][0]=2;
	errors[34][1]=52;
	errors[34][2]=28;
	errors[34][3]=20;
	errors[34][4]=61;
	*/
}